use von_neumann_probes::generation::*;
use von_neumann_probes::neural_net::NeuralNet;
use von_neumann_probes::universe::Universe;

use rand::prelude::*;
use rayon::prelude::*;

const POP_SIZE: usize = 1_000;
const SELECTION_SIZE: usize = 100;

fn main() {
	println!("gen\tavg\tbest");

	let mut gen = Generation::random(POP_SIZE);

	// 100 generations
	for gen_count in (gen.num + 1).. {
		let mut brains = sim_brains(gen.brains);
		brains.sort_by(|a, b| a.1.partial_cmp(&b.1).unwrap());

		let best = &brains[(POP_SIZE - SELECTION_SIZE)..];

		let avg: f32 = best.iter().map(|(_, b)| *b).sum::<f32>() / best.len() as f32;
		println!("{}\t{}\t{}", gen_count, avg, best.last().unwrap().1);

		// Reproduce
		let next_gen_brains: Vec<_> = (0..POP_SIZE)
			.into_iter()
			.map(|_| {
				let parent1 = best.choose(&mut rand::thread_rng()).unwrap();
				let parent2 = best.choose(&mut rand::thread_rng()).unwrap();

				parent1.0.reproduce(&parent2.0)
			})
			.collect();

		gen = Generation::new(gen_count, next_gen_brains);

		if gen_count % 10 == 0 {
			gen.save(&format!("gen-{}.pool", gen_count));
			// NOTE this is the best brain from the previous generation
			let brain = brains.last().unwrap();
			brain.0.save(&format!("best-{}.brain", gen_count));
		}
	}

	let mut brains = sim_brains(gen.brains);
	brains.sort_by(|a, b| a.1.partial_cmp(&b.1).unwrap());

	let best_brain = brains.last().unwrap();

	println!("Saving best: {}", best_brain.1);

	best_brain.0.save("best.brain");
}

fn sim_brains(brains: Vec<NeuralNet>) -> Vec<(NeuralNet, f32)> {
	brains
		.into_par_iter()
		.map(|brain| {
			const COUNT: usize = 1;

			let sum: f32 = (0..COUNT)
				.map(|_| {
					let mut u = Universe::new_with_brain(brain.clone(), 1);

					for _ in 0..1000 {
						if !u.step(1.0) {
							break;
						}
					}

					u.fitness()
				})
				.sum();
			let fitness = sum / COUNT as f32;
			(brain, fitness)
		})
		.collect()
}
