extern crate kiss3d;
extern crate nalgebra as na;

use kiss3d::light::Light;
use kiss3d::window::Window;
use na::{Translation, UnitQuaternion, Vector3};

use von_neumann_probes::neural_net::*;
use von_neumann_probes::universe::*;

fn main() {
	let mut window = Window::new("Kiss3d");
	let brain = NeuralNet::load("best-130.brain");
	let mut universe = Universe::new_with_window_and_brain(&mut window, brain, 1);

	for planet in &universe.planets {
		let mut c = window.add_sphere(planet.radius as f32);
		c.set_color(0.0, 0.8, 0.0);
		c.append_translation(&Translation::from(planet.pos));
	}

	window.set_light(Light::StickToCamera);

	for i in 0..1_000 {
		println!("{}", i);
		println!("Mass: {}", universe.probes[0].mass);
		println!(
			"Dist: {}",
			(universe.probes[0].pos - universe.planets[0].pos).norm()
		);
		if !(window.render() && universe.step(1.0)) {
			break;
		}
	}

	println!("Fitness: {}", universe.fitness());
}
