use crate::neural_net::*;
use crate::universe::*;
use rayon::prelude::*;
use serde::{Deserialize, Serialize};
use std::fs::File;
use std::io::BufWriter;
use std::io::{Read, Write};

#[derive(Serialize, Deserialize)]
pub struct Generation {
	pub num: usize,
	pub brains: Vec<NeuralNet>,
}

impl Generation {
	pub fn new(num: usize, brains: Vec<NeuralNet>) -> Self {
		Self { num, brains }
	}

	pub fn random(count: usize) -> Self {
		let brains = (0..count)
			.into_par_iter()
			.map(|_| Universe::new(None, 1).brain)
			.collect();

		Self { num: 0, brains }
	}

	pub fn save(&self, filename: &str) {
		let mut buffer = BufWriter::new(File::create(filename).unwrap());

		buffer
			.write_all(&bincode::serialize(&self).unwrap())
			.unwrap();
		buffer.flush().unwrap();
	}

	pub fn load(filename: &str) -> Self {
		let mut buffer = Vec::new();
		File::open(filename)
			.unwrap()
			.read_to_end(&mut buffer)
			.unwrap();
		bincode::deserialize(&buffer).unwrap()
	}
}
